@@ -172,13 +172,20 @@ subview<eT>::inplace_op(const Base<eT,T1>& in, const char* identifier)
       }
     else  // not a row vector
       {
-      for(uword ucol=0; ucol < s_n_cols; ++ucol)
+      if( (is_same_type<op_type, op_internal_equ>::yes) && (s.aux_row1 == 0) && (s_n_rows == s.m.n_rows) )
         {
-        if(is_same_type<op_type, op_internal_equ  >::yes)  { arrayops::copy         ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
-        if(is_same_type<op_type, op_internal_plus >::yes)  { arrayops::inplace_plus ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
-        if(is_same_type<op_type, op_internal_minus>::yes)  { arrayops::inplace_minus( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
-        if(is_same_type<op_type, op_internal_schur>::yes)  { arrayops::inplace_mul  ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
-        if(is_same_type<op_type, op_internal_div  >::yes)  { arrayops::inplace_div  ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+        arrayops::copy( s.colptr(0), B.memptr(), s.n_elem );
+        }
+      else
+        {
+        for(uword ucol=0; ucol < s_n_cols; ++ucol)
+          {
+          if(is_same_type<op_type, op_internal_equ  >::yes)  { arrayops::copy         ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+          if(is_same_type<op_type, op_internal_plus >::yes)  { arrayops::inplace_plus ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+          if(is_same_type<op_type, op_internal_minus>::yes)  { arrayops::inplace_minus( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+          if(is_same_type<op_type, op_internal_schur>::yes)  { arrayops::inplace_mul  ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+          if(is_same_type<op_type, op_internal_div  >::yes)  { arrayops::inplace_div  ( s.colptr(ucol), B.colptr(ucol), s_n_rows ); }
+          }
         }
       }
     }
@@ -913,6 +920,26 @@ subview<eT>::replace(const eT old_val, const eT new_val)
 
 
 
+template<typename eT>
+inline
+void
+subview<eT>::clean(const typename get_pod_type<eT>::result threshold)
+  {
+  arma_extra_debug_sigprint();
+  
+  subview<eT>& s = *this;
+  
+  const uword s_n_cols = s.n_cols;
+  const uword s_n_rows = s.n_rows;
+  
+  for(uword ucol=0; ucol < s_n_cols; ++ucol)
+    {
+    arrayops::clean( s.colptr(ucol), s_n_rows, threshold );
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -947,6 +974,13 @@ subview<eT>::fill(const eT val)
     }
   else
     {
+    if( (s.aux_row1 == 0) && (s_n_rows == s.m.n_rows) )
+      {
+      arrayops::inplace_set( s.colptr(0), val, s.n_elem );
+      
+      return;
+      }
+    
     for(uword ucol=0; ucol < s_n_cols; ++ucol)
       {
       arrayops::inplace_set( s.colptr(ucol), val, s_n_rows );
@@ -1368,6 +1402,13 @@ subview<eT>::extract(Mat<eT>& out, const subview<eT>& in)
     {
     arma_extra_debug_print("subview::extract(): general submatrix");
     
+    if( (in.aux_row1 == 0) && (n_rows == in.m.n_rows) )
+      {
+      arrayops::copy( out.memptr(), in.colptr(0), in.n_elem );
+      
+      return;
+      }
+    
     for(uword col=0; col < n_cols; ++col)
       {
       arrayops::copy( out.colptr(col), in.colptr(col), n_rows );
